/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * ibus - The Input Bus
 *
 * Copyright (c) 2007-2011 Peng Huang <shawn.p.huang@gmail.com>
 * Copyright (c) 2007-2011 Takao Fujiwara <takao.fjiwara1@gmail.com>
 * Copyright (c) 2007-2011 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

const Clutter = imports.gi.Clutter;
const St = imports.gi.St;
const IBus = imports.gi.IBus;
const Lang = imports.lang;
const Signals = imports.signals;
const Gettext = imports.gettext.domain('ibus10');
const _ = Gettext.gettext;

const Lightbox = imports.ui.lightbox;
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;

const common = imports._ibus.common;

const DIALOG_FADE_TIME = 0.1;

function ShellEngineAbout(enginedesc) {
    this._init(enginedesc);
}

ShellEngineAbout.prototype = {
    _init : function(enginedesc) {
        this._engine_desc = enginedesc;
        this._isOpen = false;
        this._init_ui();
    },

    _init_ui: function() {
        this._group = new Clutter.Group({ visible: false,
                                          x: 0, y: 0 });
        Main.uiGroup.add_actor(this._group);

        this._lightbox = new Lightbox.Lightbox(this._group,
                                               { inhibitEvents: true });

        this._whole_box = new St.Bin({ x_align: St.Align.MIDDLE,
                                       y_align: St.Align.MIDDLE });
        this._whole_box.connect('key-press-event',
                                Lang.bind(this, function(o, e) {
            let symbol = e.get_key_symbol();
            if (symbol == Clutter.Escape || symbol == Clutter.Return) {
                this.close();
                return true;
            }
            return false;
        }));

        this._group.add_actor(this._whole_box);
        this._lightbox.highlight(this._whole_box);

        let dialogBox = new St.BoxLayout({ style_class: 'modal-dialog',
                                           vertical: true });

        this._whole_box.set_child(dialogBox);

        let label = new St.Label({ style_class: 'run-dialog-label',
                                   text: "\n" });
        dialogBox.add(label, { expand: true, y_fill: false });

        let icon = new St.Icon({ icon_name: this._engine_desc.get_icon(),
                                 icon_type: St.IconType.SYMBOLIC,
                                 icon_size: 24 });
        dialogBox.add(icon, { expand: true, x_align: St.Align.START,
                              x_fill: false, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: "\n" +
                                     this._engine_desc.get_longname() +
                                     "\n" });
        dialogBox.add(label, { expand: true, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: _("Language: ") +
                                     IBus.get_language_name(this._engine_desc.get_language()) +
                                     "\n" });
        dialogBox.add(label, { expand: true, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: _("Keyboard layout: ") +
                                     this._engine_desc.get_layout() +
                                     "\n" });
        dialogBox.add(label, { expand: true, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: _("Author: ") +
                                     this._engine_desc.get_author() +
                                     "\n" });
        dialogBox.add(label, { expand: true, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: _("Description:\n") });
        dialogBox.add(label, { expand: true, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: this._engine_desc.get_description() });
        dialogBox.add(label, { expand: true, y_fill: false });

        this._action_box = new St.BoxLayout();

        dialogBox.add(this._action_box, { expand: true });

        let close = new St.Button({ style_class: 'window-close' });
        close.connect('clicked', Lang.bind(this, function () {
            this.close();
        }));

        this._action_box.add(close);
    },

    run : function() {
        if (this._isOpen) {
            return;
        }

        if (!Main.pushModal(this._group)) {
            return;
        }

        let monitor = global.get_focus_monitor();

        this._whole_box.set_position(monitor.x, monitor.y);
        this._whole_box.set_size(monitor.width, monitor.height);

        this._isOpen = true;
        this._lightbox.show();
        this._group.opacity = 0;
        this._group.show();

        Tweener.addTween(this._group,
                         { opacity: 255,
                           time: DIALOG_FADE_TIME,
                           transition: 'easeOutQuad'
                         });

        global.stage.set_key_focus(this._whole_box);
    },

    close : function() {
        if (!this._isOpen)
            return;

        this._isOpen = false;

        Main.popModal(this._group);

        Tweener.addTween(this._group,
                         { opacity: 0,
                           time: DIALOG_FADE_TIME,
                           transition: 'easeOutQuad',
                           onComplete: Lang.bind(this, function() {
                               this._action_box.hide();
                               this._group.hide();
                           })
                         });
    },

    destroy : function() {
        /*
        Main.popModal(this._group);
        this._whole_box.destroy();
        this._lightbox.destroy();
        this._group.destroy();
        */
    },
};
Signals.addSignalMethods(ShellEngineAbout.prototype);

function ShellPanelAbout() {
    this._init();
}

ShellPanelAbout.prototype = {
    _init : function() {
        this._isOpen = false;
        this._init_ui();
    },

    _init_ui: function() {
        this._group = new Clutter.Group({ visible: false,
                                          x: 0, y: 0 });
        Main.uiGroup.add_actor(this._group);

        this._lightbox = new Lightbox.Lightbox(this._group,
                                               { inhibitEvents: true });

        this._whole_box = new St.Bin({ x_align: St.Align.MIDDLE,
                                       y_align: St.Align.MIDDLE });
        this._whole_box.connect('key-press-event',
                                Lang.bind(this, function(o, e) {
            let symbol = e.get_key_symbol();
            if (symbol == Clutter.Escape || symbol == Clutter.Return) {
                this.close();
                return true;
            }
            return false;
        }));

        this._group.add_actor(this._whole_box);
        this._lightbox.highlight(this._whole_box);

        let dialogBox = new St.BoxLayout({ style_class: 'modal-dialog',
                                           vertical: true });

        this._whole_box.set_child(dialogBox);

        let label = new St.Label({ style_class: 'run-dialog-label',
                                   text: "\n" });
        dialogBox.add(label, { expand: true, y_fill: false });

        let icon = new St.Icon({ icon_name: "ibus",
                                 icon_type: St.IconType.SYMBOLIC,
                                 icon_size: 24 });

        dialogBox.add(icon, { expand: true, x_fill: false, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: "\n" +
                                     "IBus " +
                                     common.get_version() +
                                     "\n" });
        dialogBox.add(label, { expand: true, x_align: St.Align.MIDDLE,
                               x_fill: false, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: _("IBus is an intelligent input bus for Linux/Unix.") +
                                     "\n" });
        dialogBox.add(label, { expand: true, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: common.get_copyright() +
                                     "\n" });
        dialogBox.add(label, { expand: true, x_align: St.Align.MIDDLE,
                               x_fill: false, y_fill: false });

        label = new St.Label({ style_class: 'run-dialog-label',
                               text: "http://code.google.com/p/ibus" +
                                     "\n" });
        dialogBox.add(label, { expand: true, x_align: St.Align.MIDDLE,
                               x_fill: false, y_fill: false });

        this._action_box = new St.BoxLayout();

        dialogBox.add(this._action_box, { expand: true });

        let close = new St.Button({ style_class: 'window-close' });
        close.connect('clicked', Lang.bind(this, function () {
            this.close();
        }));

        this._action_box.add(close);
    },

    run : function() {
        if (this._isOpen) {
            return;
        }

        if (!Main.pushModal(this._group)) {
            return;
        }

        let monitor = global.get_focus_monitor();

        this._whole_box.set_position(monitor.x, monitor.y);
        this._whole_box.set_size(monitor.width, monitor.height);

        this._isOpen = true;
        this._lightbox.show();
        this._group.opacity = 0;
        this._group.show();
        Tweener.addTween(this._group,
                         { opacity: 255,
                           time: DIALOG_FADE_TIME,
                           transition: 'easeOutQuad'
                         });

        global.stage.set_key_focus(this._whole_box);
    },

    close : function() {
        if (!this._isOpen)
            return;

        this._isOpen = false;

        Main.popModal(this._group);

        Tweener.addTween(this._group,
                         { opacity: 0,
                           time: DIALOG_FADE_TIME,
                           transition: 'easeOutQuad',
                           onComplete: Lang.bind(this, function() {
                               this._action_box.hide();
                               this._group.hide();
                           })
                         });
    },

    destroy : function() {
        /*
        Main.popModal(this._group);
        this._whole_box.destroy();
        this._lightbox.destroy();
        this._group.destroy();
        */
    },
};
Signals.addSignalMethods(ShellPanelAbout.prototype);

